package com.sooka.sponest.mobile.log;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.utils.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestMapping;

import java.lang.reflect.Method;
import java.util.Arrays;

@Aspect //@Aspect注解是 Spring AOP（Aspect-Oriented Programming，面向方面编程）框架的一部分，用于标识一个类为一个切面（Aspect）
@Component //@Component注解是 Spring 框架的一部分，用于标识一个类为 Spring 组件。
public class FeignLogAspect {

    private Logger logger = LoggerFactory.getLogger(FeignLogAspect.class);

    /**
     * 在执行被@logFeignCall注解的方法或在logFeignCall注解的类内部执行的方法之前，执行一些额外的逻辑。
     * @Before 是 Spring AOP（Aspect-Oriented Programming，面向方面编程）中的一个注解，用于在方法执行之前执行一些额外的逻辑。
     * @within(logFeignCall): 这是一个条件，表示在执行被@logFeignCall注解的类内部的方法时触发@Before注解的逻辑。
     * @annotation(logFeignCall): 这是一个条件，表示在执行被@logFeignCall注解的方法时触发@Before注
     * @param joinPoint 是 Spring AOP（Aspect-Oriented Programming，面向方面编程）框架中的一个接口，用于表示被拦截的方法执行点。
     * @param logFeignCall 调用的注解
     */
    @Before("@within(logFeignCall) || @annotation(logFeignCall)")
    public void logFeignCall(JoinPoint joinPoint, LogFeignCall logFeignCall) {
        // 获取requestPath注解值，用于记录请求路径
        String requestPath = logFeignCall.requestPath();
        if (StringUtils.isEmpty(requestPath)) {
            // 如果请求路径为空, 则尝试获取特定类型请求注解中的 value 值
            requestPath = getRequestPathFromFeignAnnotation(joinPoint);
        }
        // 获取方法参数
        Object[] args = joinPoint.getArgs();

        // 记录请求信息
        logger.info("1. 路径:->" + requestPath);
        logger.info("2. 请求方式：" +logFeignCall.requestType() + "请求参数: " + Arrays.toString(args));
    }

    /**
     * @AfterReturning：这是一个 Spring AOP 注解，表示在方法执行后返回结果时触发的环绕通知（around advice）。它可以在方法成功执行后，捕获返回值并执行相应的逻辑。
     * value = "@annotation(logFeignCall)"：这部分指定了一个条件表达式，只有当当前方法上存在@logFeignCall注解时，才会触发@AfterReturning的逻辑。
     * returning = "response"：这部分指定了返回值的名称，在这里将返回值命名为response。这样在@AfterReturning的逻辑中，可以通过response来访问方法的返回结果。
     * @param joinPoint
     * @param logFeignCall
     * @param response
     */
    @AfterReturning(value = "@within(logFeignCall) ||@annotation(logFeignCall)", returning = "response")
    public void logFeignResponse(JoinPoint joinPoint, LogFeignCall logFeignCall, JSONObject response) {
        // 获取请求路径
        String requestPath = logFeignCall.requestPath();
        if (StringUtils.isEmpty(requestPath)) {
            // 如果请求路径为空, 则尝试获取 @RequestMapping 注解中的 value 值
            requestPath = getRequestPathFromFeignAnnotation(joinPoint);
        }
        // 记录响应信息
        logger.info("3. 路径->" + requestPath + "的响应:->" + response.toJSONString());
    }

    @AfterThrowing(value = "@within(logFeignCall) ||@annotation(logFeignCall)", throwing = "ex")
    public void logFeignException(JoinPoint joinPoint, LogFeignCall logFeignCall, Exception ex) {
        // 获取请求路径
        String requestPath = logFeignCall.requestPath();
        if (StringUtils.isEmpty(requestPath)) {
            // 如果请求路径为空, 则尝试获取 @RequestMapping 注解中的 value 值
            requestPath = getRequestPathFromFeignAnnotation(joinPoint);
        }

        // 记录异常信息
        logger.error("路径 " + requestPath + " 调用发生异常: " + ex.getMessage());
    }

    private String getRequestPathFromFeignAnnotation(JoinPoint joinPoint) {
        // 获取方法签名对象
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        // 获取方法对象
        Method method = signature.getMethod();
        // 尝试获取 PostMapping 注解
        PostMapping postMapping = AnnotationUtils.findAnnotation(method, PostMapping.class);
        if (postMapping != null) {
            // 如果找到了 PostMapping 注解，返回 value 属性中的第一个值
            return postMapping.value()[0];
        }
        // 尝试获取 PutMapping 注解
        PutMapping putMapping = AnnotationUtils.findAnnotation(method, PutMapping.class);
        if (putMapping != null) {
            // 如果找到了 PutMapping 注解，返回 value 属性中的第一个值
            return putMapping.value()[0];
        }
        // 尝试获取 GetMapping 注解
        GetMapping getMapping = AnnotationUtils.findAnnotation(method, GetMapping.class);
        if (getMapping != null) {
            // 如果找到了 GetMapping 注解，返回 value 属性中的第一个值
            return getMapping.value()[0];
        }
        // 如果以上特定类型的请求注解都不符合，则尝试获取 @RequestMapping 注解中的 value 值
        RequestMapping requestMapping = AnnotationUtils.findAnnotation(method, RequestMapping.class);
        if (requestMapping != null) {
            // 获取 @RequestMapping 注解的 value 属性值数组
            String[] value = requestMapping.value();
            // 如果 value 属性值数组的长度大于 0，则返回第一个值
            if (value.length > 0) {
                return value[0];
            }
        }
        // 如果没有找到任何请求注解，则返回空字符串
        return "";
    }
}
